@@ -7,7 +7,7 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
 use Fcntl;
 use integer;
 
-$VERSION = '0.11';
+$VERSION = '0.12';
 
 require Exporter;
 require DynaLoader;
@@ -37,24 +37,21 @@ sub new {
 	$alg =~ s/\D+//g if defined $alg;
 	$alg =~ s/^3?(0|224|256|384|512)$/$1/ if defined $alg;
 	if (ref($class)) {	# instance method
-		unless (defined($alg) && ($alg != $class->algorithm)) {
+		if (!defined($alg) || ($alg == $class->algorithm)) {
 			sharewind($$class);
 			return($class);
 		}
-		if ($$class) { shaclose($$class); $$class = undef }
-		return unless $$class = shaopen($alg);
-		return($class);
+		return shainit($$class, $alg) ? $class : undef;
 	}
 	$alg = 224 unless defined $alg;
 	my $state = shaopen($alg) || return;
 	my $self = \$state;
 	bless($self, $class);
-	return($self);
 }
 
 sub DESTROY {
 	my $self = shift;
-	if ($$self) { shaclose($$self); $$self = undef }
+	shaclose($$self);
 }
 
 sub clone {
@@ -86,30 +83,33 @@ sub _bail {
         Carp::croak($msg);
 }
 
-sub _addfile {  # this is "addfile" from Digest::base 1.00
-    my ($self, $handle) = @_;
+my $_can_T_filehandle;
 
-    my $n;
-    my $buf = "";
+sub _istext {
+	local *FH = shift;
+	my $file = shift;
 
-    while (($n = read($handle, $buf, 4096))) {
-        $self->add($buf);
-    }
-    _bail("Read failed") unless defined $n;
-
-    $self;
+	if (! defined $_can_T_filehandle) {
+		local $^W = 0;
+		my $istext = eval { -T FH };
+		$_can_T_filehandle = $@ ? 0 : 1;
+		return $_can_T_filehandle ? $istext : -T $file;
+	}
+	return $_can_T_filehandle ? -T FH : -T $file;
 }
 
 sub Addfile {
 	my ($self, $file, $mode) = @_;
 
-	return(_addfile($self, $file)) unless ref(\$file) eq 'SCALAR';
+	return(_addfilebin($self, $file)) unless ref(\$file) eq 'SCALAR';
 
 	$mode = defined($mode) ? $mode : "";
-	my ($binary, $portable, $BITS) = map { $_ eq $mode } ("b", "p", "0");
+	my ($binary, $UNIVERSAL, $BITS, $portable) =
+		map { $_ eq $mode } ("b", "U", "0", "p");
 
 		## Always interpret "-" to mean STDIN; otherwise use
 		## sysopen to handle full range of POSIX file names
+
 	local *FH;
 	$file eq '-' and open(FH, '< -')
 		or sysopen(FH, $file, O_RDONLY)
@@ -126,28 +126,18 @@ sub Addfile {
 		return($self);
 	}
 
-	binmode(FH) if $binary || $portable;
-	unless ($portable && -T $file) {
-		$self->_addfile(*FH);
-		close(FH);
-		return($self);
+	binmode(FH) if $binary || $portable || $UNIVERSAL;
+	if ($UNIVERSAL && _istext(*FH, $file)) {
+		$self->_addfileuniv(*FH);
 	}
-
-	my ($n1, $n2);
-	my ($buf1, $buf2) = ("", "");
-
-	while (($n1 = read(FH, $buf1, 4096))) {
-		while (substr($buf1, -1) eq "\015") {
-			$n2 = read(FH, $buf2, 4096);
-			_bail("Read failed") unless defined $n2;
-			last unless $n2;
-			$buf1 .= $buf2;
+	elsif ($portable && _istext(*FH, $file)) {
+		while (<FH>) {
+			s/\015?\015\012/\012/g;
+			s/\015/\012/g;
+			$self->add($_);
 		}
-		$buf1 =~ s/\015?\015\012/\012/g;	# DOS/Windows
-		$buf1 =~ s/\015/\012/g;			# early MacOS
-		$self->add($buf1);
 	}
-	_bail("Read failed") unless defined $n1;
+	else { $self->_addfilebin(*FH) }
 	close(FH);
 
 	$self;
@@ -467,15 +457,24 @@ argument to one of the following values:
 
 	"b"	read file in binary mode
 
-	"p"	use portable mode
+	"U"	use universal newlines
 
 	"0"	use BITS mode
 
-The "p" mode ensures that the digest value of I<$filename> will be the
-same when computed on different operating systems.  It accomplishes
-this by internally translating all newlines in text files to UNIX format
-before calculating the digest.  Binary files are read in raw mode with
-no translation whatsoever.
+	"p"	use portable mode (to be deprecated)
+
+The "U" mode is modeled on Python's "Universal Newlines" concept, whereby
+DOS and Mac OS line terminators are converted internally to UNIX newlines
+before processing.  This ensures consistent digest values when working
+simultaneously across multiple file systems.  B<The "U" mode influences
+only text files>, namely those passing Perl's I<-T> test; binary files
+are processed with no translation whatsoever.
+
+The "p" mode differs from "U" only in that it treats "\r\r\n" as a single
+newline, a quirky feature designed to accommodate legacy applications that
+occasionally added an extra carriage return before DOS line terminators.
+The "p" mode will be phased out eventually in favor of the cleaner and
+more well-established Universal Newlines concept.
 
 The BITS mode ("0") interprets the contents of I<$filename> as a logical
 stream of bits, where each ASCII '0' or '1' character represents a 0 or
